package com.elab.spring.config;

import com.elab.core.aop.annotations.EnableElabDB;
import com.elab.mq.listener.AbstractMessageListener;
import com.elab.mq.listener.ConsumerInterceptor;
import com.elab.mq.msg.IMsgProducerFacade;
import com.elab.mq.msg.ProducerInterceptor;
import com.elab.spring.dao.IConsumerDao;
import com.elab.spring.dao.IProducerDao;
import com.elab.spring.intercept.RMQCacheConsumerInterceptor;
import com.elab.spring.intercept.RMQCacheProducerInterceptor;
import com.elab.spring.intercept.RMQMsgTableConsumerInterceptor;
import com.elab.spring.intercept.RMQMsgTableProducerInterceptor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Module TODO
 * @Description TODO
 * @Author liukaixiong
 * @Date 2020/12/30 15:15
 */
@Configuration
@EnableElabDB(basePackageClasses = {IConsumerDao.class, IProducerDao.class})
public class RmqConfiguration {

    @Bean
    @ConditionalOnMissingBean(value = {ConsumerInterceptor.class})
//    @ConditionalOnBean(value = {CacheTemplate.class, AbstractMessageListener.class})
    public ConsumerInterceptor cacheConsumerInterceptor() {
        return new RMQCacheConsumerInterceptor();
    }

    @Bean
    @ConditionalOnMissingBean(value = {ProducerInterceptor.class})
//    @ConditionalOnBean(value = {CacheTemplate.class, IMsgProducerFacade.class})
    public ProducerInterceptor cacProducerInterceptor() {
        return new RMQCacheProducerInterceptor();
    }

    @Bean
    @ConditionalOnMissingBean(value = {ConsumerInterceptor.class})
    @ConditionalOnBean(value = {AbstractMessageListener.class})
    public ConsumerInterceptor msgConsumerInterceptor() {
        return new RMQMsgTableConsumerInterceptor();
    }

    @Bean
    @ConditionalOnMissingBean(value = {ProducerInterceptor.class})
    @ConditionalOnBean(value = {IMsgProducerFacade.class})
    public ProducerInterceptor msgProducerInterceptor() {
        return new RMQMsgTableProducerInterceptor();
    }

}
