package com.elab.spring.intercept;

import com.aliyun.openservices.ons.api.SendResult;
import com.elab.mq.model.MessageModel;
import com.elab.mq.msg.ProducerInterceptor;
import com.elab.redis.CacheTemplate;
import com.jay.monitor.data.client.MonitorSendProducer;
import com.jay.monitor.data.client.utils.MonitorUtils;
import com.jay.monitor.data.core.model.serializable.MQDataDTO;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @Module rmq拦截器
 * @Description 基于缓存实现
 * @Author liukaixiong
 * @Date 2020/12/30 17:25
 */
public class RMQCacheProducerInterceptor implements ProducerInterceptor {

    /**
     * 所有客户端共用这一个唯一编号
     */
    private String defaultPrefix = "RMQProducer:index";

    @Autowired
    private CacheTemplate cacheTemplate;

    @Override
    public boolean check(MessageModel messageModel) {
        // 由于多个客户端发送，如果区分开的话就得维护多个
        String cacheKey = getCacheKey();
        Long producerId = cacheTemplate.string().increment(cacheKey, 1);
        messageModel.setProducerId(producerId.toString());
        return true;
    }

    @Override
    public void success(MessageModel messageModel, SendResult result) {
        sendMonitorData(messageModel, 1);
    }

    @Override
    public void error(MessageModel messageModel, Throwable e) {
        sendMonitorData(messageModel, -1);
    }

    private void sendMonitorData(MessageModel data, Integer status) {
        String content = (String) data.getObject(String.class);
        MQDataDTO mqDataDTO = MonitorUtils.builderProducerRMQDataDTO(data.getTopic(), data.getKey(), data.getTag(), data.getMsgID(), content, status);
        MonitorSendProducer.sendMsg(data.getTopic() + "-" + data.getTag(), mqDataDTO);
    }


    private String getCacheKey() {
//        String prefixWith = elabRedisProperties.getPrefixWith() == null ? "unknown" : elabRedisProperties.getPrefixWith();
//        StringBuilder key = new StringBuilder();
//        key.append(prefixWith).append(":");
//        key.append(defaultPrefix);
        return defaultPrefix;
    }
}
